@@ -16,10 +16,8 @@
  *    -stibor-
  */
 #include "config.h"
-#include "ylib.h"
 #include "wmapp.h"
 #include "applet.h"
-#include "ypointer.h"
 #include "acpustatus.h"
 
 #if IWM_STATES
@@ -66,20 +64,105 @@
 
 extern ref<YPixmap> taskbackPixmap;
 
+#if __linux__
+class YStat {
+public:
+    int count() {
+        int cpus = 0;
+        if (have()) {
+            char* p = data;
+            while (p && (p = strstr(p, "\ncpu")) != nullptr) {
+                cpus += ASCII::isDigit(p[4]);
+                p = strchr(p + 4, '\n');
+            }
+        }
+        return cpus;
+    }
+    void recycle() {
+        if (data) {
+            *data = '\0';
+        }
+    }
+    char* cpustatus(int cpuid) {
+        char* status = nullptr;
+        if (have()) {
+            if (cpuid == -1) {
+                char* p = strstr(data, "cpu ");
+                if (p == data || p[-1] == '\n') {
+                    status = p + 4;
+                }
+            } else {
+                char buf[32];
+                snprintf(buf, sizeof buf, "\ncpu%d ", cpuid);
+                char* p = strstr(data, buf);
+                if (p) {
+                    status = p + strlen(buf);
+                }
+            }
+        }
+        return status;
+    }
+    YStat() : fildes(-1), data(nullptr) { }
+    ~YStat() {
+        if (fildes > 2) close(fildes);
+        if (data) delete[] data;
+    }
+private:
+    int fildes;
+    char* data;
+    const size_t size = 8192;
+    bool have() { return nonempty(data) || read(); }
+    bool read() {
+        const char path[] = "/proc/stat";
+        if (fildes >= 0 && lseek(fildes, 0, SEEK_SET) < 0) {
+            if (ONCE)
+                fail("seek %s", path);
+            close(fildes);
+            fildes = -1;
+        }
+        if (fildes < 0 && (fildes = open(path, O_RDONLY | O_CLOEXEC)) < 0) {
+            if (ONCE)
+                fail("open %s", path);
+            return false;
+        }
+        if (data == nullptr) {
+            data = new char[size];
+            *data = '\0';
+        }
+        ssize_t len = ::read(fildes, data, size - 1);
+        if (len <= 0) {
+            if (ONCE)
+                fail("read %s", path);
+            close(fildes);
+            fildes = -1;
+            delete[] data;
+            data = nullptr;
+            return false;
+        } else {
+            data[len] = '\0';
+        }
+        return true;
+    }
+};
+#endif
+
 YFont CPUStatus::tempFont;
 YTemp* CPUStatus::fTemp;
 
-CPUStatus::CPUStatus(YWindow *aParent, CPUStatusHandler *aHandler, int cpuid) :
-    IApplet(this, aParent),
+CPUStatus::CPUStatus(YWindow* parent, CPUStatusHandler* handler,
+                     int cpuid, YStat* stat) :
+    IApplet(this, parent),
     fCpuID(cpuid),
     statusUpdateCount(0),
     unchanged(taskBarCPUSamples),
     cpu(taskBarCPUSamples, IWM_STATES),
-    fHandler(aHandler),
+    fHandler(handler),
+    fStat(stat),
+    fBackground(None),
     fTempColor(&clrCpuTemp)
 {
     cpu.clear();
-    for (int a(0); a < taskBarCPUSamples; a++) {
+    for (int a = 0; a < taskBarCPUSamples; a++) {
         cpu[a][IWM_IDLE] = 1;
     }
     memset(last_cpu, 0, sizeof(last_cpu));
@@ -95,19 +178,20 @@ CPUStatus::CPUStatus(YWindow *aParent, CPUStatusHandler *aHandler, int cpuid) :
     setSize(taskBarCPUSamples, taskBarGraphHeight);
     getStatus();
     updateStatus();
-    char buf[99];
-    snprintf(buf, 99, "CPU%d", cpuid);
+    char buf[48];
+    snprintf(buf, 48, "CPU%d", cpuid);
     setTitle(buf);
 }
 
 CPUStatus::~CPUStatus() {
+    if (fBackground) {
+        XFreePixmap(xapp->display(), fBackground);
+    }
 }
 
 bool CPUStatus::picture() {
     bool change = (hasPixmap() == false);
 
-    Graphics G(getPixmap(), width(), height(), depth());
-
 #if __linux__
     if (cpustatusShowAcpiTempInGraph) {
         change = true;
@@ -115,6 +199,29 @@ bool CPUStatus::picture() {
     }
 #endif
 
+    if (color[IWM_IDLE] == false) {
+        if (fGeometry != geometry()) {
+            if (fBackground) {
+                XFreePixmap(xapp->display(), fBackground);
+            }
+            fBackground = createPixmap();
+            fGeometry = geometry();
+
+            Graphics gfx(fBackground, width(), height(), depth());
+            ref<YImage> gradient(getGradient());
+            if (gradient != null)
+                gfx.drawImage(gradient,
+                              x(), y(), width(), height(), 0, 0);
+            else if (taskbackPixmap != null)
+                gfx.fillPixmap(taskbackPixmap,
+                               0, 0, width(), height(), x(), y());
+            else
+                gfx.clear();
+        }
+    }
+
+    Graphics G(getPixmap(), width(), height(), depth());
+
     if (change) {
         unchanged = 0;
         fill(G);
@@ -139,15 +246,7 @@ void CPUStatus::fill(Graphics& g) {
         g.setColor(color[IWM_IDLE]);
         g.fillRect(0, 0, width(), height());
     } else {
-        ref<YImage> gradient(getGradient());
-
-        if (gradient != null)
-            g.drawImage(gradient,
-                        x(), y(), width(), height(), 0, 0);
-        else
-            if (taskbackPixmap != null)
-                g.fillPixmap(taskbackPixmap,
-                             0, 0, width(), height(), x(), y());
+        g.copyDrawable(fBackground, 0, 0, width(), height(), 0, 0);
     }
 }
 
@@ -161,7 +260,7 @@ void CPUStatus::draw(Graphics& g) {
     statusUpdateCount = 0;
 
     for (int i = first; i < limit; i++) {
-        unsigned long long
+        cpubytes
             user    = cpu[i][IWM_USER],
             nice    = cpu[i][IWM_NICE],
             sys     = cpu[i][IWM_SYS],
@@ -175,7 +274,7 @@ void CPUStatus::draw(Graphics& g) {
         int y = height() - 1;
 
         if (total > 1) { /* better show 0 % CPU than nonsense on startup */
-            unsigned long long
+            cpubytes
                 intrbar, sysbar, nicebar, userbar,
                 iowaitbar, softirqbar, stealbar, totalbar = 0,
                 round = total / h / 2;  /* compute also with rounding errs */
@@ -243,15 +342,7 @@ void CPUStatus::draw(Graphics& g) {
                 g.setColor(color[IWM_IDLE]);
                 g.drawLine(i, 0, i, y);
             } else {
-                ref<YImage> gradient(getGradient());
-
-                if (gradient != null)
-                    g.drawImage(gradient,
-                                 this->x() + i, this->y(), width(), y + 1, i, 0);
-                else
-                    if (taskbackPixmap != null)
-                        g.fillPixmap(taskbackPixmap,
-                                     i, 0, width(), y + 1, this->x() + i, this->y());
+                g.copyDrawable(fBackground, i, 0, width() - i, y + 1, i, 0);
             }
         }
     }
@@ -275,15 +366,19 @@ public:
             if (temp) {
                 int cent = temp / 1000;
                 int hund = temp % 1000 / 100;
-                snprintf(tempbuf + done, buflen - done, "%d.%d ", cent, hund);
+                if (best && ! hund) {
+                    snprintf(tempbuf + done, buflen - done, "%d ", cent);
+                } else {
+                    snprintf(tempbuf + done, buflen - done, "%d.%d ", cent, hund);
+                }
                 done += strlen(tempbuf + done);
                 if (best) {
                     break;
                 }
             }
         }
-        if (done && done + int(strlen(centigrade)) + 1 < buflen && ! best) {
-            strlcpy(tempbuf + done, centigrade, buflen - done);
+        if (done && done + int(strlen(centigrade)) + 1 < buflen) {
+            strlcpy(tempbuf + done - best, centigrade, buflen - done);
         }
         return done > 0;
     }
@@ -302,7 +397,9 @@ private:
         if (strlcat(path, "/temp", sizeof path) < sizeof path) {
             int fd = open(path, O_RDONLY);
             if (fd > 2) {
-                read(fd, buf, sizeof buf);
+                if (read(fd, buf, sizeof buf) < 1) {
+                    buf[0] = '\0';
+                }
                 close(fd);
             }
         }
@@ -327,7 +424,9 @@ private:
             int fd = openat(dirfd, buf, O_RDONLY);
             if (fd > 2) {
                 memset(buf, 0, sizeof buf);
-                read(fd, buf, sizeof buf - 1);
+                if (read(fd, buf, sizeof buf - 1) < 1) {
+                    buf[0] = '\0';
+                }
                 close(fd);
                 char* newl = strchr(buf, '\n');
                 if (newl) {
@@ -404,7 +503,7 @@ void CPUStatus::temperature(Graphics& g) {
 #if __linux__
     if (cpustatusShowAcpiTempInGraph) {
         char temp[10];
-        getAcpiTemp(temp, sizeof(temp));
+        getAcpiTemp(temp, sizeof(temp), true);
         g.setColor(fTempColor);
         if (tempFont == null)
             tempFont = tempFontName;
@@ -413,8 +512,22 @@ void CPUStatus::temperature(Graphics& g) {
             int h = height();
             int y = (h - 1 - tempFont->height()) / 2 + tempFont->ascent();
             int w = tempFont->textWidth(temp);
+            if (w > int(width())) {
+                int s = strspn(temp, "0123456789.");
+                if (s && temp[s]) {
+                    temp[s] = '\0';
+                    w = tempFont->textWidth(temp);
+                }
+                while (w > int(width()) && strchr(temp, '.') && 0 < s) {
+                    temp[--s] = '\0';
+                    if (temp[s-1] == '.') {
+                        temp[--s] = '\0';
+                    }
+                    w = tempFont->textWidth(temp);
+                }
+            }
             int x = max(0, (int(g.rwidth()) - w) / 2);
-            g.drawChars(temp, 0, 3, x, y);
+            g.drawString(x, y, temp);
         }
     }
 #endif
@@ -564,27 +677,24 @@ void CPUStatus::handleClick(const XButtonEvent &up, int count) {
 }
 
 void CPUStatus::updateStatus() {
-    for (int i(1); i < taskBarCPUSamples; i++)
-        cpu.copyTo(i, i - 1);
+    cpu.shiftLeft();
     getStatus();
     repaint();
 }
 
 int CPUStatus::getAcpiTemp(char *tempbuf, int buflen, bool best) {
-    int retbuflen = 0;
-
     tempbuf[0] = '\0';
 
 #if __linux__
     if (fTemp == nullptr) {
         fTemp = new YTemp;
     }
-    if (fTemp && fTemp->read_temp(tempbuf, buflen, best)) {
-        retbuflen += int(strlen(tempbuf));
+    if (fTemp) {
+        fTemp->read_temp(tempbuf, buflen, best);
     }
 #endif
 
-    return retbuflen;
+    return int(strlen(tempbuf));
 }
 
 float CPUStatus::getCpuFreq(int cpu) {
@@ -621,62 +731,36 @@ float CPUStatus::getCpuFreq(int cpu) {
 }
 
 
-void CPUStatus::getStatusPlatform() {
+void CPUStatus::getStatusLinux() {
 #ifdef __linux__
-    char *p = nullptr, buf[4096], *end = nullptr;
-    unsigned long long cur[IWM_STATES];
-    int s;
-
-    fileptr fd(fopen("/proc/stat", "r"));
-    if (fd == nullptr)
+    char* line = fStat->cpustatus(fCpuID);
+    if (line == nullptr)
         return;
 
-    while (fgets(buf, sizeof buf, fd) && 0 == strncmp(buf, "cpu", 3)) {
-        if (fCpuID == -1) {
-            if (ASCII::isSpaceOrTab(buf[3]))
-                p = buf + 4;
-            break;
-        }
-        if (ASCII::isDigit(buf[3]) && strtol(buf + 3, &end, 10) == fCpuID) {
-            if (end > buf && ASCII::isSpaceOrTab(*end)) {
-                p = 1 + end;
-                break;
-            }
+    cpubytes cur[IWM_STATES];
+    char* p = line;
+    for (int i = 0; i < IWM_STATES; ++i) {
+        cur[i] = 0;
+        while (*p == ' ' || *p == '\t')
+            ++p;
+        if (*p == '+')
+            ++p;
+        if ('0' <= *p && *p <= '9') {
+            cur[i] = (*p++ - '0');
+            while ('0' <= *p && *p <= '9')
+                cur[i] = (10 * cur[i]) + (*p++ - '0');
         }
     }
-    fd.close();
-    if (p == nullptr)
-        return;
-
-    s = sscanf(p, "%llu %llu %llu %llu %llu %llu %llu %llu",
-               &cur[IWM_USER],    &cur[IWM_NICE],
-               &cur[IWM_SYS],     &cur[IWM_IDLE],
-               &cur[IWM_IOWAIT],  &cur[IWM_INTR],
-               &cur[IWM_SOFTIRQ], &cur[IWM_STEAL]);
-    switch (s) {
-    case 4:
-        /* Linux 2.4 */
-        cur[IWM_INTR]    =
-        cur[IWM_IOWAIT]  =
-        cur[IWM_SOFTIRQ] = 0;
-        /* FALL THROUGH */
-    case 7:
-        /* Linux < 2.6.11 */
-        cur[IWM_STEAL] = 0;
-        /* FALL THROUGH */
-    case 8:
-        break;
-    default:
-        return;
-    }
-    int i = 0;
-    for (i = 0; i < IWM_STATES; i++) {
+    for (int i = 0; i < IWM_STATES; i++) {
         cpu[taskBarCPUSamples - 1][i] = cur[i] - last_cpu[i];
         last_cpu[i] = cur[i];
     }
+#endif
+}
 
-    return;
-
+void CPUStatus::getStatusPlatform() {
+#if __linux__
+    getStatusLinux();
 #elif __OpenBSD__ || __NetBSD__ || __FreeBSD__
 
 #if defined __NetBSD__
@@ -708,14 +792,14 @@ void CPUStatus::getStatusPlatform() {
     }
 #endif
 
-    unsigned long long cur[IWM_STATES];
+    cpubytes cur[IWM_STATES];
     cur[IWM_USER]    = cp_time[CP_USER];
     cur[IWM_NICE]    = cp_time[CP_NICE];
     cur[IWM_SYS]     = cp_time[CP_SYS];
-    cur[IWM_INTR]    = cp_time[CP_INTR];
-    cur[IWM_IOWAIT]  = 0;
-    cur[IWM_SOFTIRQ] = 0;
     cur[IWM_IDLE]    = cp_time[CP_IDLE];
+    cur[IWM_IOWAIT]  = 0;
+    cur[IWM_INTR]    = cp_time[CP_INTR];
+    cur[IWM_SOFTIRQ] = 0;
     cur[IWM_STEAL]   = 0;
 
     for (int i = 0; i < IWM_STATES; i++) {
@@ -753,6 +837,7 @@ CPUStatusControl::CPUStatusControl(YSMListener *smActionListener,
     smActionListener(smActionListener),
     iapp(iapp),
     aParent(aParent),
+    fStat(nullptr),
     fMenuCPU(-1),
     fSamples(taskBarCPUSamples),
     fPid(0)
@@ -761,20 +846,26 @@ CPUStatusControl::CPUStatusControl(YSMListener *smActionListener,
     fUpdateTimer.setTimer(taskBarCPUDelay, this, true);
 }
 
+CPUStatusControl::~CPUStatusControl() {
+    CPUStatus::freeFont();
+#if __linux__
+    CPUStatus::freeTemp();
+    delete fStat;
+#endif
+}
+
 void CPUStatusControl::GetCPUStatus(bool combine) {
     int count = 0;
 #if __linux__
-    if (combine == false) {
-        fileptr fd("/proc/stat", "r");
-        if (fd) {
-            char buf[128];
-            while (fgets(buf, sizeof buf, fd) && 0 == memcmp(buf, "cpu", 3))
-                count += ASCII::isDigit(buf[3]);
-        }
+    if (fStat == nullptr) {
+        fStat = new YStat();
+    }
+    if (combine == false && fStat) {
+        count = fStat->count();
     }
 #endif
     do {
-        fCPUStatus += new CPUStatus(aParent, this, --count);
+        fCPUStatus += new CPUStatus(aParent, this, --count, fStat);
     } while (0 < count);
 }
 
@@ -792,6 +883,11 @@ bool CPUStatusControl::handleTimer(YTimer* timer) {
             GetCPUStatus(cpus < 2);
             iapp->relayout();
         }
+#if __linux__
+        if (fStat) {
+            fStat->recycle();
+        }
+#endif
         for (CPUStatus* status : fCPUStatus) {
             status->handleTimer(timer);
         }
