@@ -1,5 +1,5 @@
-#ifndef __MAILBOX_H
-#define __MAILBOX_H
+#ifndef AMAILBOX_H
+#define AMAILBOX_H
 
 #include "ywindow.h"
 #include "ytimer.h"
@@ -58,7 +58,7 @@ public:
 
     void startCheck();
     void startSSL();
-    cstring inbox();
+    mstring inbox();
     int portNumber();
     void setState(ProtocolState newState);
 
@@ -70,7 +70,7 @@ public:
     void parseImap();
 
     int write(const char* buf, int len = 0);
-    int write(const cstring& str);
+    int write(mstring str);
     void error(mstring str);
     void release();
     bool ssl() const;
@@ -101,6 +101,8 @@ private:
     bool fTrace;
     mstring fReason;
     static int fInstanceCounter;
+    static int fDestructCounter;
+    static csmart openssl_path;
 
     void resolve();
     void countMessages();
@@ -110,8 +112,7 @@ private:
 
 class MailBoxStatus:
     public IApplet,
-    private Picturer,
-    private YTimerListener
+    private Picturer
 {
 public:
     enum MailBoxState {
@@ -127,8 +128,8 @@ public:
     virtual ~MailBoxStatus();
 
     virtual void handleClick(const XButtonEvent &up, int count);
-    virtual void handleCrossing(const XCrossingEvent &crossing);
 
+    int checkDelay() const;
     void checkMail();
     void mailChecked(MailBoxState mst, long count, long unread);
     void newMailArrived(long count, long unread);
@@ -137,7 +138,6 @@ public:
 
 private:
     virtual bool picture();
-    virtual bool handleTimer(YTimer *t);
     virtual void updateToolTip();
     ref<YPixmap> statePixmap();
     void draw(Graphics& g);
@@ -145,14 +145,17 @@ private:
     MailBoxState fOldState;
     MailBoxState fState;
     MailCheck check;
-    lazy<YTimer> fMailboxCheckTimer;
     MailHandler *fHandler;
     long fCount;
     long fUnread;
     bool fSuspended;
 };
 
-class MailBoxControl : public MailHandler, private YActionListener {
+class MailBoxControl :
+    public MailHandler,
+    private YTimerListener,
+    private YActionListener
+{
 public:
     MailBoxControl(IApp *app, YSMListener *smActionListener,
                    IAppletContainer *taskBar, YWindow *aParent);
@@ -164,11 +167,12 @@ private:
 
     virtual void actionPerformed(YAction button, unsigned int modifiers);
     virtual void handleClick(const XButtonEvent &up, MailBoxStatus *client);
+    virtual bool handleTimer(YTimer *t);
     virtual void runCommandOnce(const char *resource, const char *cmdline);
     virtual void runCommand(const char *cmdline);
 
     typedef YObjectArray<MailBoxStatus> ArrayType;
-    ArrayType fMailBoxStatus;
+    ArrayType fMailBoxes;
 
 public:
     IApp *app;
@@ -177,10 +181,14 @@ public:
     YWindow *aParent;
     osmart<YMenu> fMenu;
     MailBoxStatus *fMenuClient;
+    lazy<YTimer> fCheckTimer;
     long fPid;
+    int fCount;
+    int fDelay;
+    int fDelta;
 
     typedef ArrayType::IterType IterType;
-    IterType iterator() { return fMailBoxStatus.reverseIterator(); }
+    IterType iterator() { return fMailBoxes.reverseIterator(); }
 };
 
 #endif
