@@ -10,23 +10,58 @@
 #include "wmapp.h"
 #include "applet.h"
 #include "amemstatus.h"
+
+#if MEM_STATES
+
 #include "ymenuitem.h"
 #include "sysdep.h"
 #include "default.h"
 #include "ytimer.h"
 #include "intl.h"
 
-#if MEM_STATES
-
-#define USE_PROC_MEMINFO
-
 extern ref<YPixmap> taskbackPixmap;
 
+bool MemInfo::read() {
+    const char path[] = "/proc/meminfo";
+
+    if (fildes >= 0 && lseek(fildes, 0, SEEK_SET) < 0) {
+        if (ONCE)
+            fail("seek %s", path);
+        close(fildes);
+        fildes = -1;
+    }
+    if (fildes < 0 && (fildes = open(path, O_RDONLY | O_CLOEXEC)) < 0) {
+        if (ONCE)
+            fail("open %s", path);
+        return false;
+    }
+
+    if (buf == nullptr) {
+        buf = new char[size];
+        *buf = '\0';
+    }
+
+    len = ::read(fildes, buf, size - 1);
+    if (len <= 0) {
+        if (ONCE)
+            fail("read %s", path);
+        close(fildes);
+        fildes = -1;
+        delete[] buf;
+        buf = nullptr;
+    } else {
+        buf[len] = '\0';
+    }
+    return buf && len;
+}
+
 MEMStatus::MEMStatus(IAppletContainer* taskBar, YWindow *aParent):
     IApplet(this, aParent),
     samples(taskBarMEMSamples, MEM_STATES),
     statusUpdateCount(0),
     unchanged(taskBarMEMSamples),
+    fBackground(None),
+    fMemInfo(new MemInfo),
     taskBar(taskBar)
 {
     fUpdateTimer->setTimer(taskBarMEMDelay, this, true);
@@ -48,11 +83,39 @@ MEMStatus::MEMStatus(IAppletContainer* taskBar, YWindow *aParent):
 }
 
 MEMStatus::~MEMStatus() {
+    if (fBackground) {
+        XFreePixmap(xapp->display(), fBackground);
+    }
+    delete fMemInfo;
 }
 
 bool MEMStatus::picture() {
     bool create = (hasPixmap() == false);
 
+    for (int i = 0; i < MEM_STATES; ++i) {
+        if (color[i] == false) {
+            if (fGeometry != geometry()) {
+                if (fBackground) {
+                    XFreePixmap(xapp->display(), fBackground);
+                }
+                fBackground = createPixmap();
+                fGeometry = geometry();
+
+                Graphics gfx(fBackground, width(), height(), depth());
+                ref<YImage> gradient(getGradient());
+                if (gradient != null)
+                    gfx.drawImage(gradient,
+                                  x(), y(), width(), height(), 0, 0);
+                else if (taskbackPixmap != null)
+                    gfx.fillPixmap(taskbackPixmap,
+                                   0, 0, width(), height(), x(), y());
+                else
+                    gfx.clear();
+            }
+            break;
+        }
+    }
+
     Graphics G(getPixmap(), width(), height(), depth());
 
     if (create)
@@ -67,15 +130,7 @@ void MEMStatus::fill(Graphics& g) {
         g.setColor(color[MEM_FREE]);
         g.fillRect(0, 0, width(), height());
     } else {
-        ref<YImage> gradient(getGradient());
-
-        if (gradient != null)
-            g.drawImage(gradient,
-                        x(), y(), width(), height(), 0, 0);
-        else
-            if (taskbackPixmap != null)
-                g.fillPixmap(taskbackPixmap,
-                             0, 0, width(), height(), x(), y());
+        g.copyDrawable(fBackground, 0, 0, width(), height(), 0, 0);
     }
 }
 
@@ -94,7 +149,7 @@ void MEMStatus::draw(Graphics& g) {
         int y = h;
         for (int j = 0; j < MEM_STATES; j++) {
             int bar;
-            if (j == MEM_STATES-1) {
+            if (j == MEM_STATES - 1) {
                 bar = y;
             } else {
                 bar = (int)((h * samples[i][j]) / total);
@@ -106,21 +161,10 @@ void MEMStatus::draw(Graphics& g) {
 
             if (color[j]) {
                 g.setColor(color[j]);
-                g.drawLine(i, y-1, i, y-bar);
+                g.drawLine(i, y - 1, i, y - bar);
             } else {
-                ref<YImage> gradient(getGradient());
-
-                if (gradient != null)
-                    g.drawImage(gradient,
-                                this->x() + i, this->y() + y - bar,
-                                width(), bar,
-                                i, y - bar);
-                else
-                    if (taskbackPixmap != null)
-                        g.fillPixmap(taskbackPixmap,
-                                     i, y - bar,
-                                     width(), bar,
-                                     this->x() + i, this->y() + y - bar);
+                g.copyDrawable(fBackground,
+                               i, y - bar, width() - i, bar, i, y - bar);
             }
             y -= bar;
         }
@@ -137,30 +181,32 @@ bool MEMStatus::handleTimer(YTimer *t) {
 }
 
 void MEMStatus::printAmount(char *out, size_t outSize,
-                            unsigned long long amount) {
-    if (amount >= (200ull*1024*1024*1024)) {
+                            membytes amount) {
+    if (amount >> 30 >= 200) {
         snprintf( out, outSize, "%llu %.20s",
-                  amount/(1024*1024*1024),
+                  amount >> 30,
                   _("GB") );
-    } else if (amount >= (200*1024*1024)) {
+    }
+    else if (amount >> 20 >= 200) {
         snprintf( out, outSize, "%llu %.20s",
-                  amount/(1024*1024),
+                  amount >> 20,
                   _("MB") );
-    } else if (amount >= (200*1024)) {
+    }
+    else if (amount >> 10 >= 200) {
         snprintf( out, outSize, "%llu %.20s",
-                  amount/1024,
+                  amount >> 10,
                   _("kB") );
     } else {
         snprintf( out, outSize, "%llu %.20s",
                   amount,
                   _("bytes") );
     }
-    out[outSize-1]='\0';
+    out[outSize - 1] = '\0';
 }
 
 void MEMStatus::updateToolTip() {
-    membytes* cur = samples[taskBarMEMSamples-1];
-    membytes total = samples.sum(taskBarMEMSamples-1);
+    membytes* cur = samples[taskBarMEMSamples - 1];
+    membytes total = samples.sum(taskBarMEMSamples - 1);
 
     char totalStr[64];
     printAmount(totalStr, sizeof(totalStr), total);
@@ -184,62 +230,55 @@ void MEMStatus::updateToolTip() {
 }
 
 void MEMStatus::updateStatus() {
-    for (int i(1); i < taskBarMEMSamples; i++) {
-        samples.copyTo(i, i - 1);
-    }
+    samples.shiftLeft();
     getStatus();
     repaint();
 }
 
-unsigned long long MEMStatus::parseField(const char *buf, size_t bufLen,
-                                         const char *needle) {
-#ifdef USE_PROC_MEMINFO
+membytes MemInfo::parseField(const char* needle) {
     ptrdiff_t needleLen = strlen(needle);
     for (const char* str(buf); str && (str = strstr(str, needle)) != nullptr; ) {
         if (str == buf || str[-1] == '\n') {
-            char *endptr = nullptr;
-            membytes result = strtoull(str+needleLen, &endptr, 10);
+            char* endptr = nullptr;
+            membytes result = strtoull(str + needleLen, &endptr, 10);
 
             while (*endptr != 0 && *endptr == ' ')
                 endptr++;
 
-            if (*endptr=='k') {   // normal case
+            if (*endptr == 'k') {   // normal case
                 result *= 1024;
-            } else if (*endptr=='M') {
+            } else if (*endptr == 'M') {
                 result *= 1024*1024;
-            } else if (*endptr=='G') {
+            } else if (*endptr == 'G') {
                 result *= 1024*1024*1024;
             }
             return result;
         }
         str = strchr(str + needleLen, '\n');
     }
-#endif /*USE_PROC_MEMINFO*/
     return 0;
 }
 
 void MEMStatus::getStatus() {
-    membytes *cur = samples[taskBarMEMSamples-1];
-    samples.clear(taskBarMEMSamples-1);
+    membytes *cur = samples[taskBarMEMSamples - 1];
+    samples.clear(taskBarMEMSamples - 1);
     cur[MEM_FREE] = 1;
 
-#ifdef USE_PROC_MEMINFO
-    char buf[4096];
-    auto len = filereader("/proc/meminfo").read_all(BUFNSIZE(buf));
-    if (len > 0) {
-        cur[MEM_BUFFERS] = parseField(buf, len, "Buffers:");
-        cur[MEM_CACHED] = parseField(buf, len, "Cached:");
-        cur[MEM_FREE] = parseField(buf, len, "MemFree:");
-
-        membytes total = max(1ULL, parseField(buf, len, "MemTotal:"));
-
+    fMemInfo->recycle();
+    if (fMemInfo->read()) {
+        cur[MEM_BUFFERS] = fMemInfo->parseField("Buffers:");
+        cur[MEM_CACHED] = fMemInfo->parseField("Cached:");
+        cur[MEM_FREE] = fMemInfo->parseField("MemFree:");
+        membytes total = fMemInfo->parseField("MemTotal:");
+        if (total < 1)
+            total = 1;
         membytes user = total;
         for (int j = 0; j < MEM_STATES; j++) {
             user -= cur[j];
         }
         cur[MEM_USER] = user;
     }
-#endif // USE_PROC_MEMINFO
+    fMemInfo->discard();
 
     ++statusUpdateCount;
     int last = taskBarMEMSamples - 1;
@@ -247,7 +286,7 @@ void MEMStatus::getStatus() {
     unchanged = same ? 1 + unchanged : 0;
 }
 
-void MEMStatus::actionPerformed(YAction action, unsigned int modifiers) {
+void MEMStatus::actionPerformed(YAction action, unsigned modifiers) {
     if (action == actionClose) {
         hide();
         taskBar->relayout();
