@@ -96,7 +96,7 @@ private:
     void updateItems();
 
     YColorName bg, fg;
-    ref<YFont> font;
+    YFont font;
     int fontWidth, fontHeight;
 };
 
@@ -155,7 +155,10 @@ YIconView::YIconView(YScrollView *view, YWindow *aParent):
 {
     fView = view;
 
-    font = YFont::getFont("-b&h-lucida-medium-r-*-*-*-120-*-*-*-*-*-*", "monospace:size=10");
+    static const char* corefont = "-b&h-lucida-medium-r-*-*-*-120-*-*-*-*-*-*";
+    static const char* xftefont = "monospace:size=10";
+    YFontName fontName(&corefont, &xftefont);
+    font = fontName;
     fontWidth = font->textWidth("M");
     fontHeight = font->height();
 
@@ -327,10 +330,10 @@ bool YIconView::handleKey(const XKeyEvent& key) {
     if (fVerticalScroll->handleScrollKeys(key) == true) {
         return true;
     }
-    if (key.type == KeyPress) {
+    if (key.type == KeyRelease) {
         KeySym k = keyCodeToKeySym(key.keycode);
         int m = KEY_MODMASK(key.state);
-        if (k == XK_Escape) {
+        if (k == XK_Escape && m == 0) {
             xapp->exit(0);
             return true;
         }
@@ -550,7 +553,7 @@ void ObjectList::updateList() {
     adir dir(fPath);
     YArray<ObjectIconItem *> file;
     while (dir.next()) {
-        ObjectIconItem *o = new ObjectIconItem(dir.path(), dir.entry());
+        ObjectIconItem *o = new ObjectIconItem(fPath, dir.entry());
         if (o) {
             if (o->isFolder())
                 list->addItem(o);
@@ -599,7 +602,7 @@ int main(int argc, char **argv) {
     bindtextdomain(PACKAGE, LOCDIR);
     textdomain(PACKAGE);
 
-    check_argv(argc, argv, help(), VERSION);
+    check_argv(argc, argv, help, VERSION);
 
     YStringArray names;
     YXApplication app(&argc, &argv);
