@@ -4,6 +4,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <langinfo.h>
 
 char const *ApplicationName("testlocale");
 bool multiByte(true);
@@ -16,21 +17,21 @@ foreign_str(char const *charset, char const *foreign)
     size_t size = len + 80;
     char * str = new char [size];
 
-    snprintf(str, size, "\033%%/1\\%03zo\\%03zo%s\002%s",
-             128 + len / 128, 128 + len % 128, charset, foreign);
+    snprintf(str, size, "%s: %s",
+             charset, foreign);
 
     return str;
 }
 
 static void
-print_string(const YLChar *lstr, YUChar *ustr)
+print_string(const char *lstr, wchar_t *ustr, size_t ulen)
 {
     printf ("In locale encoding: \"%s\"\n", lstr);
     printf ("In unicode encoding: \"");
 
-    for (YUChar * u(ustr); *u; ++u) printf("\\U%04x", *u);
+    for (wchar_t * u(ustr); *u; ++u) printf("\\U%04x", *u);
 
-    puts ("\"");
+    printf ("\" (len=%zu)\n", ulen);
 }
 
 #define TEST_RATING(LocaleFragment) \
@@ -41,26 +42,38 @@ print_string(const YLChar *lstr, YUChar *ustr)
 
 int main() {
 #ifdef CONFIG_I18N
+    {
+        YLocale locale;
+        printf("Default locale: %s\n", YLocale::getLocaleName());
+        printf("nl_langinfo(%d): %s\n", CODESET, nl_langinfo(CODESET));
+    }
+
     size_t ulen;
+    wchar_t* ustr;
 
-    const YLChar *lstr("Mhrenkuter");
-    YUChar *ustr(YLocale("de_DE.iso-8859-1").
-                 unicodeString(lstr, strlen(lstr), ulen));
-    print_string(lstr, ustr);
+    {
+        const char* lstr = "Mhrenkuter";
+        YLocale locale("de_DE.iso-8859-1");
+        ulen = 0;
+        ustr = locale.unicodeString(lstr, strlen(lstr), ulen);
+        print_string(lstr, ustr, ulen);
 
-    lstr = foreign_str ("ISO8859-15", "Euro sign: ");
-    ustr = YLocale("de_DE.iso-8859-1").
-        unicodeString(lstr, strlen(lstr), ulen);
-    print_string(lstr, ustr);
+        size_t llen = 0;
+        char * cstr = locale.localeString(ustr, ulen, llen);
+        printf("cstr: %s (%zu)\n", cstr, llen);
+    }
 
+    {
+        const char* lstr = foreign_str ("ISO8859-15", "Euro sign: ");
+        YLocale locale("de_DE.iso-8859-1");
+        ulen = 0;
+        ustr = locale.unicodeString(lstr, strlen(lstr), ulen);
+        print_string(lstr, ustr, ulen);
 
-/*
-    YLChar * utf8(YLocale("de_DE.utf8").localeString(unicode));
-    printf("utf8: %s\n", utf8);
-
-    YLChar * latin1(YLocale("de_DE.iso-8859-1").localeString(unicode));
-    printf("iso-8859-1: %s\n", latin1);
-*/
+        size_t llen = 0;
+        char * cstr = locale.localeString(ustr, ulen, llen);
+        printf("cstr: %s (%zu)\n", cstr, llen);
+    }
 
     {
         YLocale locale("de_DE@euro");
