@@ -5,13 +5,10 @@
 
 // themable preferences (themes can set only these)
 
-#if defined(CONFIG_I18N) || 1 // maybe not such a good idea
 XIV(bool, prettyClock,                          false)
-#else
-XIV(bool, prettyClock,                          true)
-#endif
 XIV(bool, rolloverTitleButtons,                 false)
 
+XIV(int, taskBarGraphHeight,                    20)
 XIV(unsigned, taskbuttonIconOffset,             0)
 XIV(unsigned, trayIconMaxWidth,                 32)
 XIV(unsigned, trayIconMaxHeight,                24)
@@ -91,6 +88,7 @@ XSV(const char *, clrMoveSizeStatus,            "rgb:C0/C0/C0")
 XSV(const char *, clrMoveSizeStatusText,        "rgb:00/00/00")
 XSV(const char *, clrQuickSwitch,               "rgb:C0/C0/C0")
 XSV(const char *, clrQuickSwitchText,           "rgb:00/00/00")
+XSV(const char *, clrQuickSwitchBorder,         0)
 XSV(const char *, clrQuickSwitchActive,         0)
 XSV(const char *, clrDefaultTaskBar,            "rgb:C0/C0/C0")
 XSV(const char *, clrNormalButton,              "rgb:C0/C0/C0")
@@ -99,6 +97,7 @@ XSV(const char *, clrActiveButton,              "rgb:E0/E0/E0")
 XSV(const char *, clrActiveButtonText,          "rgb:00/00/00")
 XSV(const char *, clrToolButton,                "")
 XSV(const char *, clrToolButtonText,            "")
+XSV(const char *, clrKeyboardLayoutText,        "")
 XSV(const char *, clrWorkspaceActiveButton,     "")
 XSV(const char *, clrWorkspaceActiveButtonText, "")
 XSV(const char *, clrWorkspaceNormalButton,     "")
@@ -157,10 +156,15 @@ cfoption icewm_themable_preferences[] = {
     OBV("RolloverButtonsSupported",             &rolloverTitleButtons,          "Does it support the 'O' title bar button images (for mouse rollover)"),
     OBV("TaskBarClockLeds",                     &prettyClock,                   "Task bar clock/battery monitor uses nice pixmap LCD display (but then it doesn't display correctly in many languages anymore, e.g., for Japanese and Korean it works only when a real font is used and not the LCD pixmaps"),
 
+    OIV("TaskBarGraphHeight",                   &taskBarGraphHeight, 16, 1000,  "Height of taskbar monitoring applets"),
     OUV("TaskbuttonIconOffset",                 &taskbuttonIconOffset, 0, 16,   "Width of taskbutton side icons"),
     OUV("TrayIconMaxWidth",                     &trayIconMaxWidth, 16, 128,     "Maximum scaled width of tray icons"),
     OUV("TrayIconMaxHeight",                    &trayIconMaxHeight, 16, 128,    "Maximum scaled height of tray icons"),
     OBV("TrayDrawBevel",                        &trayDrawBevel,                 "Surround the tray with plastic border"),
+    OIV("TaskBarApmGraphWidth",                 &taskBarApmGraphWidth, 1, 1000,  "Width of battery Monitor"),
+    OIV("TaskBarCPUSamples",                    &taskBarCPUSamples, 2, 1000,    "The width of the CPU Monitor applet in pixels"),
+    OIV("TaskBarMEMSamples",                    &taskBarMEMSamples, 2, 1000,    "The width of the Memory Monitor applet in pixels"),
+    OIV("TaskBarNetSamples",                    &taskBarNetSamples, 2, 1000,    "The width of the Net Monitor applet in pixels"),
 
     OBV("TitleBarCentered",                     &titleBarCentered,              "Draw window title centered (obsoleted by TitleBarJustify)"),
     OBV("TitleBarJoinLeft",                     &titleBarJoinLeft,              "Join title*S and title*T"),
@@ -257,6 +261,7 @@ cfoption icewm_themable_preferences[] = {
     OSV("ColorMoveSizeStatusText",              &clrMoveSizeStatusText,         "Text color of move/resize status window"),
 
     OSV("ColorQuickSwitch",                     &clrQuickSwitch,                "Background of the quick switch window"),
+    OSV("ColorQuickSwitchBorder",               &clrQuickSwitchBorder,          "Border color of the quick switch window"),
     OSV("ColorQuickSwitchText",                 &clrQuickSwitchText,            "Text color in the quick switch window"),
     OSV("ColorQuickSwitchActive",               &clrQuickSwitchActive,          "Rectangle arround the active icon in the quick switch window"),
     OSV("ColorDefaultTaskBar",                  &clrDefaultTaskBar,             "Background of the taskbar"),
@@ -289,7 +294,8 @@ cfoption icewm_themable_preferences[] = {
     OSV("ColorInputSelectionText",              &clrInputSelectionText,         "Selected text in an entry field"),
 
     OSV("ColorClock",                           &clrClock,                      "Background of non-LCD clock, leave empty to force transparency"),
-    OSV("ColorClockText",                       &clrClockText,                  "Background of non-LCD monitor"),
+    OSV("ColorClockText",                       &clrClockText,                  "Text color of non-LCD clock"),
+    OSV("ColorKeyboardLayoutText",              &clrKeyboardLayoutText,         "Color of keyboard layout indicator"),
 
     OSV("ColorApm",                             &clrApm,                        "Background of battery monitor, leave empty to force transparency"),
     OSV("ColorApmText",                         &clrApmText,                    "Text color of battery monitor"),
