@@ -32,6 +32,7 @@ public:
     upath removeExtension() const;
     upath replaceExtension(const char *ext) const;
     mstring expand() const;
+    mstring real();
 
     bool fileExists();
     bool dirExists();
@@ -40,10 +41,12 @@ public:
     bool isReadable();
     bool isWritable();
     bool isExecutable();
+    bool isSearchable();
     bool isHttp() const;
     bool hasProtocol() const;
     int access(int mode = 0);
     int mkdir(int mode = 0700);
+    int chdir();
     int open(int flags, int mode = 0666);
     FILE* fopen(const char *mode);
     int stat(struct stat *st);
@@ -53,6 +56,7 @@ public:
     fcsmart loadText();
     bool copyFrom(upath from, int mode = 0666);
     bool testWritable(int mode = 0666);
+    bool ensureDirectory();
     int fnMatch(const char* pattern, int flags = 0);
 
     upath& operator=(const upath& p) {
@@ -89,6 +93,9 @@ public:
         return glob(path(), list, opt);
     }
 
+    static void redirectOutput(const char* outputFile);
+    static mstring cwd();
+
 private:
     mstring fPath;
 
