@@ -140,6 +140,7 @@ private:
     virtual int handleError(XErrorEvent *xev);
     virtual void keyboardRemap();
     void runRestart(const char *path, char *const *args);
+    void launchHelp(const char* file);
 
     FocusModel focusMode;
     Window managerWindow;
@@ -153,6 +154,7 @@ private:
     void initIcons();
     void initIconSize();
     void reparseKeyPrefs();
+    void toggleWindowList();
 };
 
 extern YWMApp * wmapp;
