@@ -51,11 +51,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
     }
 
     if (clientMouseActions) {
-        unsigned int k = button.button + XK_Pointer_Button1 - 1;
-        unsigned int m = KEY_MODMASK(button.state);
-        unsigned int vm = VMod(m);
-
-        if (gMouseWinSize.eq(k, vm)) {
+        if (gMouseWinSize == button) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
 
             int px = button.x + x();
@@ -84,7 +80,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
             }
             return ;
         }
-        else if (gMouseWinMove.eq(k, vm)) {
+        else if (gMouseWinMove == button) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
 
             if (getFrame()->canMove()) {
@@ -96,14 +92,15 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
             }
             return ;
         }
-        else if (gMouseWinRaise.eq(k, vm)
-            && (gMouseWinRaise != gMouseWinLower || getFrame()->canRaise()))
-        {
+        else if (gMouseWinRaise == button) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
-            getFrame()->wmRaise();
+            if (getFrame()->canRaise())
+                getFrame()->wmRaise();
+            else if (gMouseWinRaise == gMouseWinLower)
+                getFrame()->wmLower();
             return ;
         }
-        else if (gMouseWinLower.eq(k, vm)) {
+        else if (gMouseWinLower != gMouseWinRaise && gMouseWinLower == button) {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
             getFrame()->wmLower();
             return ;
@@ -170,15 +167,12 @@ void YClientContainer::regrabMouse() {
 
 void YClientContainer::grabActions() {
     if (clientMouseActions && fHaveActionGrab == false) {
-        WMKey grab[] = { gMouseWinMove, gMouseWinSize,
-                         gMouseWinRaise, gMouseWinLower };
-        const int count = int ACOUNT(grab) - (gMouseWinRaise == gMouseWinLower);
-        for (int i = 0; i < count; ++i) {
-            int button = int(grab[i].key) - XK_Pointer_Button1 + Button1;
-            if (inrange(button, Button1, Button3)) {
-                grabVButton(button, grab[i].mod);
-            }
-        }
+        const Window win = handle();
+        gMouseWinMove.grab(win);
+        gMouseWinSize.grab(win);
+        gMouseWinRaise.grab(win);
+        if (gMouseWinLower != gMouseWinRaise)
+            gMouseWinLower.grab(win);
         fHaveActionGrab = true;
     }
 }
