@@ -5,31 +5,20 @@
 
 class filereader {
 public:
-    filereader(int fd, bool closes = true) :
-        nFd(fd),
-        bCloses(closes)
-    {
-    }
-
+    filereader(int fd) : nFd(fd) { }
     filereader(const char* filename);
-
-    filereader(filereader&& rv) :
-        nFd(rv.nFd),
-        bCloses(rv.bCloses)
-    {
-        rv.nFd = -1;
-    }
-
+    filereader(filereader&& rv) : nFd(rv.nFd) { rv.nFd = -1; }
     ~filereader();
 
     int read_all(char* buf, size_t buflen);
-    fcsmart read_all();
-    fcsmart read_pipe(long timeout, bool* timedOut);
+    fcsmart read_pipe(long timeout = -1L, bool* timedOut = nullptr);
+    fcsmart read_size(size_t size);
+    static fcsmart read_path(const char* filename);
 
 private:
     int nFd;
-    bool bCloses;
 
+    fcsmart read_loop();
     filereader(const filereader&) = delete;
 };
 
