@@ -14,16 +14,18 @@ public:
     virtual void repaint();
 
     void setText(const char* label);
+    void multiline(bool enable) { fMultied = enable; }
 
 private:
     mstring fLabel;
+    bool fMultied;
     bool fPainted;
 
     void autoSize();
 
     static YColorName labelFg;
     static YColorName labelBg;
-    static ref<YFont> labelFont;
+    static YFont labelFont;
     static int labelObjectCount;
 };
 
