@@ -5,17 +5,15 @@
 
 #ifdef CONFIG_SESSION
 
-class YSMPoll: public YPoll<class YSMApplication> {
+class YSMPoll : public YPollBase {
 public:
     virtual void notifyRead();
-    virtual void notifyWrite();
-    virtual bool forRead();
-    virtual bool forWrite();
+    virtual bool forRead() { return true; }
 };
 
 class YSMApplication: public YXApplication { /// !!! should be possible without X
 public:
-    YSMApplication(int *argc, char ***argv, const char *displayName = 0);
+    YSMApplication(int *argc, char ***argv, const char *displayName = nullptr);
     virtual ~YSMApplication();
 
     bool haveSessionManager();
